@@ -25,6 +25,7 @@ sub new {
   $self->{min_workers} = $self->{max_workers} if $self->{min_workers} > $self->{max_workers};
   $self->{timeout} = $arg{timeout} if exists $arg{timeout};
   $self->{max_checkouts} = $arg{max_checkouts} if exists $arg{max_checkouts};
+  $self->{refork_after_error} = 1 if $arg{refork_after_error};
 
   $self->{total_workers} = 0;
   $self->{connecting_workers} = {};
@@ -80,7 +81,7 @@ sub populate_workers {
                                 my ($worker, $fatal, $message) = @_;
 
                                 my $checkout = $self->{workers_to_checkouts}->{0 + $worker};
-                                $checkout->throw_error('worker connection suddenly died') if $checkout;
+                                $checkout->throw_fatal_error('worker connection suddenly died') if $checkout;
 
                                 $self->destroy_worker($worker);
                                 $self->populate_workers;
@@ -136,9 +137,6 @@ sub try_to_fill_pending_checkouts {
 sub make_worker_occupied {
   my ($self, $worker) = @_;
 
-  ## Cancel the "sk" detection push_read
-  $worker->{_queue} = [];
-
   delete $self->{available_workers}->{0 + $worker};
   $self->{occupied_workers}->{0 + $worker} = $worker;
 
@@ -159,12 +157,6 @@ sub make_worker_available {
   ## Cancel any push_read callbacks installed while worker was occupied
   $worker->{_queue} = [];
 
-  ## Install an "sk" detection push_read
-  $worker->push_read( json => sub {
-    my ($handle, $response) = @_;
-    $self->destroy_worker($worker) if $response->[0] eq 'sk';
-  });
-
   delete $self->{occupied_workers}->{0 + $worker};
   $self->{available_workers}->{0 + $worker} = $worker;
 }
